@@ -15,8 +15,6 @@
 #ifndef GRPC_SRC_CORE_EXT_TRANSPORT_CHAOTIC_GOOD_SERVER_TRANSPORT_H
 #define GRPC_SRC_CORE_EXT_TRANSPORT_CHAOTIC_GOOD_SERVER_TRANSPORT_H
 
-#include <grpc/support/port_platform.h>
-
 #include <stdint.h>
 #include <stdio.h>
 
@@ -42,8 +40,10 @@
 
 #include <grpc/event_engine/event_engine.h>
 #include <grpc/event_engine/memory_allocator.h>
+#include <grpc/grpc.h>
 #include <grpc/slice.h>
 #include <grpc/support/log.h>
+#include <grpc/support/port_platform.h>
 
 #include "src/core/ext/transport/chaotic_good/chaotic_good_transport.h"
 #include "src/core/ext/transport/chaotic_good/frame.h"
@@ -56,6 +56,7 @@
 #include "src/core/lib/promise/activity.h"
 #include "src/core/lib/promise/context.h"
 #include "src/core/lib/promise/if.h"
+#include "src/core/lib/promise/inter_activity_latch.h"
 #include "src/core/lib/promise/inter_activity_pipe.h"
 #include "src/core/lib/promise/loop.h"
 #include "src/core/lib/promise/mpsc.h"
@@ -77,15 +78,14 @@
 namespace grpc_core {
 namespace chaotic_good {
 
-class ChaoticGoodServerTransport final : public Transport,
-                                         public ServerTransport {
+class ChaoticGoodServerTransport final : public ServerTransport {
  public:
   ChaoticGoodServerTransport(
-      const ChannelArgs& args,
-      std::unique_ptr<PromiseEndpoint> control_endpoint,
-      std::unique_ptr<PromiseEndpoint> data_endpoint,
+      const ChannelArgs& args, PromiseEndpoint control_endpoint,
+      PromiseEndpoint data_endpoint,
       std::shared_ptr<grpc_event_engine::experimental::EventEngine>
-          event_engine);
+          event_engine,
+      HPackParser hpack_parser, HPackCompressor hpack_encoder);
   ~ChaoticGoodServerTransport() override;
 
   FilterStackTransport* filter_stack_transport() override { return nullptr; }
@@ -94,7 +94,7 @@ class ChaoticGoodServerTransport final : public Transport,
   absl::string_view GetTransportName() const override { return "chaotic_good"; }
   void SetPollset(grpc_stream*, grpc_pollset*) override {}
   void SetPollsetSet(grpc_stream*, grpc_pollset_set*) override {}
-  void PerformOp(grpc_transport_op*) override { Crash("unimplemented"); }
+  void PerformOp(grpc_transport_op*) override;
   grpc_endpoint* GetEndpoint() override { return nullptr; }
   void Orphan() override { delete this; }
 
@@ -107,36 +107,50 @@ class ChaoticGoodServerTransport final : public Transport,
   absl::Status NewStream(uint32_t stream_id, CallInitiator call_initiator);
   absl::optional<CallInitiator> LookupStream(uint32_t stream_id);
   absl::optional<CallInitiator> ExtractStream(uint32_t stream_id);
+  auto SendCallInitialMetadataAndBody(uint32_t stream_id,
+                                      MpscSender<ServerFrame> outgoing_frames,
+                                      CallInitiator call_initiator);
+  auto SendCallBody(uint32_t stream_id, MpscSender<ServerFrame> outgoing_frames,
+                    CallInitiator call_initiator);
+  static auto SendFragment(ServerFragmentFrame frame,
+                           MpscSender<ServerFrame> outgoing_frames,
+                           CallInitiator call_initiator);
   auto CallOutboundLoop(uint32_t stream_id, CallInitiator call_initiator);
-  auto OnTransportActivityDone();
-  auto TransportReadLoop();
-  auto TransportWriteLoop();
+  auto OnTransportActivityDone(absl::string_view activity);
+  auto TransportReadLoop(RefCountedPtr<ChaoticGoodTransport> transport);
+  auto ReadOneFrame(ChaoticGoodTransport& transport);
+  auto TransportWriteLoop(RefCountedPtr<ChaoticGoodTransport> transport);
   // Read different parts of the server frame from control/data endpoints
   // based on frame header.
   // Resolves to a StatusOr<tuple<SliceBuffer, SliceBuffer>>
   auto ReadFrameBody(Slice read_buffer);
   void SendCancel(uint32_t stream_id, absl::Status why);
   auto DeserializeAndPushFragmentToNewCall(FrameHeader frame_header,
-                                           BufferPair buffers);
-  auto DeserializeAndPushFragmentToExistingCall(FrameHeader frame_header,
-                                                BufferPair buffers);
+                                           BufferPair buffers,
+                                           ChaoticGoodTransport& transport);
+  auto DeserializeAndPushFragmentToExistingCall(
+      FrameHeader frame_header, BufferPair buffers,
+      ChaoticGoodTransport& transport);
   auto MaybePushFragmentIntoCall(absl::optional<CallInitiator> call_initiator,
-                                 absl::Status error, ClientFragmentFrame frame);
+                                 absl::Status error, ClientFragmentFrame frame,
+                                 uint32_t stream_id);
   auto PushFragmentIntoCall(CallInitiator call_initiator,
-                            ClientFragmentFrame frame);
+                            ClientFragmentFrame frame, uint32_t stream_id);
 
   Acceptor* acceptor_ = nullptr;
+  InterActivityLatch<void> got_acceptor_;
   MpscReceiver<ServerFrame> outgoing_frames_;
-  ChaoticGoodTransport transport_;
   // Assigned aligned bytes from setting frame.
   size_t aligned_bytes_ = 64;
   Mutex mu_;
   // Map of stream incoming server frames, key is stream_id.
   StreamMap stream_map_ ABSL_GUARDED_BY(mu_);
+  uint32_t last_seen_new_stream_id_ = 0;
   grpc_event_engine::experimental::MemoryAllocator allocator_;
-  std::shared_ptr<grpc_event_engine::experimental::EventEngine> event_engine_;
-  ActivityPtr writer_;
-  ActivityPtr reader_;
+  ActivityPtr writer_ ABSL_GUARDED_BY(mu_);
+  ActivityPtr reader_ ABSL_GUARDED_BY(mu_);
+  ConnectivityStateTracker state_tracker_ ABSL_GUARDED_BY(mu_){
+      "chaotic_good_server", GRPC_CHANNEL_READY};
 };
 
 }  // namespace chaotic_good
